# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from os.path import join, exists
from ConfigParser import RawConfigParser

from sugar_server import env, util, datastore


class Cursor(object):

    def __init__(self):
        self._modified = False

    def insert(self, table_, **kwargs):
        self._insert(table_, **kwargs)

    def insert_or_update(self, table_, **kwargs):
        return self._insert(table_, **kwargs)

    def update(self, table_, **kwargs):
        self.insert(table_, **kwargs)

    def select(self, table_, **kwargs):
        db = _get_db()

        if not db.has_section(table_):
            return

        for __, row in db.items(table_):
            row = util.json.loads(row)
            for field, value in kwargs.items():
                if dict(row).get(field) != value:
                    break
            else:
                yield row

    def delete(self, table_, **kwargs):
        db = _get_db()

        if not db.has_section(table_):
            return

        for __, row in db.items(table_):
            row = util.json.loads(row)
            for field, value in kwargs.items():
                if dict(row).get(field) != value:
                    break
            else:
                key = datastore.TABLES[table_][0]
                db.remove_option(table_, row[key])

    def close(self):
        if self._modified:
            _set_db()
            self._modified = False

    def _insert(self, table_, **row):
        db = _get_db()

        if not db.has_section(table_):
            db.add_section(table_)

        for key, value in row.items():
            if not isinstance(value, str):
                row[key] = str(value)

        key = datastore.TABLES[table_][0]
        inserted = not db.has_option(table_, row[key])
        db.set(table_, row[key], util.json.dumps(row))
        self._modified = True

        return inserted


def get_db_mtime():
    return os.stat(_db_name()).st_mtime if exists(_db_name()) else None


_db = None


def _db_name():
    return join(env.var_path(), env.db_name.value + '.db')


def _get_db():
    global _db
    if _db is None:
        _db = RawConfigParser()
        _db.read(_db_name())
    return _db


def _set_db():
    if _db is None:
        return
    f = util.new_file(_db_name())
    _db.write(f)
    f.close()
