# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from SocketServer import TCPServer, ThreadingMixIn
from SimpleHTTPServer import SimpleHTTPRequestHandler
from gettext import gettext as _

from sugar_server import env, misc


_access_logger = None
_server = misc.ThreadingTCPServer()


def start(services):
    global _access_logger
    _access_logger = env.get_logger('httpd', '')
    _access_logger.propagate = False
    _server.start(_Server(services))


def stop():
    _server.stop()


class _Server(TCPServer, ThreadingMixIn):

    def __init__(self, services):

        class Handler(_Handler):

            def handle_method(self, method):
                mod_handler = '%s_%s' % (method, self.path.split('/')[1])
                for mod in services:
                    if not hasattr(mod, mod_handler):
                        continue
                    logging.debug(_('Found %s HTTP handler in %s'),
                            mod_handler, mod.__name__)
                    try:
                        mod.__dict__[mod_handler](self)
                    except Exception, error:
                        logging.exception(_('Cannot process %s request'),
                                self.requestline)
                        self.reply(403, str(error))
                    return True
                logging.debug(_('No %s HTTP handlers found'), mod_handler)

        self.allow_reuse_address = True
        TCPServer.__init__(self,
                (env.hostname.value, env.httpd_port.value),
                Handler)


class _Handler(SimpleHTTPRequestHandler):

    send_content = True

    def handle_method(self, request):
        assert not 'implemented'

    def reply(self, code, reply=None):
        if reply is None:
            reply_len = 0
        else:
            if isinstance(reply, str):
                reply_len = len(reply)
            else:
                reply_len = reply.tell()
                reply.seek(0)

        self.send_response(code)
        self.send_header('Content-type', 'text/plain')
        self.send_header('Content-length', reply_len)
        self.end_headers()

        if reply is not None:
            if self.send_content:
                if isinstance(reply, str):
                    self.wfile.write(reply)
                    self.wfile.flush()
                else:
                    self.copyfile(reply, self.wfile)
            if not isinstance(reply, str):
                reply.close()

    def do_HEAD(self):
        self.send_content = False
        if not self.handle_method('GET'):
            self.reply(403)

    def do_GET(self):
        if not self.handle_method('GET'):
            self.reply(403)

    def do_POST(self):
        if not self.handle_method('POST'):
            self.reply(403)

    def log_message(self, fmt, *args):
        request = fmt % args
        message = '%s - - [%s] %s' % \
                (self.address_string(), self.log_date_time_string(), request)
        _access_logger.info(message)
