# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from sugar_server.util import Option, Command, _
from sugar_server import misc


activation_port = Option(
        _('port for incomming connections for activation service'),
        default=1910, type_cast=int)
default_expiry = Option(
        _('default expiry for delegated leases in days, if integer value, ' \
                'or, exact date in format "yyyymmddThhmmssZ"'),
        default='30', type_cast=misc.Expiry)
max_expiry = Option(
        _('maximal lease expiry value in days, if integer value, or, ' \
                'exact date in format "yyyymmddThhmmssZ"'),
        default='90', type_cast=misc.Expiry)
delegation = Option(
        _('delegate leases if there are no existed ones'),
        default=True, type_cast=bool, action='store_true')

import_dir = Command(
        _('import files generated by bios-crypto utility: lease.sig, ' \
          'leases.dat, d-lease.sig, d-oats.sig; files will be imported ' \
          'from the PATH'),
        'PATH')
import_lease = Command(
        _('import leases in JSON format or one text lease per line, leases ' \
          'will be imported from the FILE or stdin'),
        '[FILE]')
import_lease_delegation = Command(
        _('import lease delegations, one text delegation per line, ' \
          'delegations will be imported from the FILE or stdin'),
        '[FILE]')
import_oats_delegation = Command(
        _('import oats delegations, one text delegation per line, ' \
          'delegations will be imported from the FILE or stdin'),
        '[FILE]')


def hashed_path(serial, *args):
    from sugar_server import env
    return env.service_root(*(list(args) + [serial[-2:], serial]))


def lease_path(serial):
    return hashed_path(serial, 'leases')


def oats_delegations_path(serial):
    return hashed_path(serial, 'oats-delegations')


def lease_delegations_path(serial):
    return hashed_path(serial, 'lease-delegations')
