# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pwd
from SimpleXMLRPCServer import SimpleXMLRPCServer
from gettext import gettext as _

from sugar_server import env, misc


etc = env.import_from('etc')
directory = env.import_from('directory')

_logger = env.get_logger('id')
_service = misc.ThreadingTCPServer()


def setup():
    directory.setup()


def start():
    server = SimpleXMLRPCServer(
            (env.hostname.value, etc.id_port.value),
            logRequests=False)
    server.register_function(register_handler, 'register')
    _service.start(server)


def stop():
    _service.stop()
    directory.close()


def register_handler(serial, nickname, uuid, pubkey):
    _logger.info('Got reqistration request serial="%s" nickname="%s", ' \
            'uuid="%s" pubkey="%s"', serial, nickname.encode('utf-8'),
            uuid, pubkey)

    try:
        directory.register(uuid, nickname, pubkey, serial)

        _logger.info('Student successfully registered')

        me = pwd.getpwuid(os.getuid()).pw_name
        return {'success': 'OK',
                'backupurl': '%s@%s:/' % (me, env.hostname.value),
                'jabberserver': etc.jabber.value,
                'backuppath': '/',
                }
    except Exception, error:
        _logger.exception(_('Cannot process reqistration request'))
        return {'success': 'ERR', 'error': str(error)}
